#!/usr/bin/env python

# LED Scrolling Sign Serial Driver
# Created for the TweetSign
#
# Andy Creeth and Ajay Dhesikan
# Advised by Rick Brown
#
# Updated: 11/1/2011

# Inspired by the MIT Center for Civic Media
# Lost In Boston Realtime


# LIBRARIES
import serial
import os
import sys

class ledSign:
    
    # The following constants were taken from the Lost In Boston Realtime
    # LED Sign Driver: https://github.com/c4fcm/Realtime-Community-Sign
    
	BAUDRATE = 9600
    
	# header constants for comms to the sign
	COMM_TEXT_FILE_NAME_0 = ['0']
	COMM_CMD_START = ['\x00','\x00','\x00','\x00','\x00']
	COMM_START_OF_HEAD = ['\x01']
	COMM_SEND_ADDR_PC = ['F','F']
	COMM_RECEIVER_ADDR_BCAST = ['0','0']
	COMM_START_OF_TEXT = ['\x02']

	# commands for the sign
	COMM_CMD_WRITE_TEXT = ['A']
	COMM_CMD_WRITE_VAR = ['C']
	COMM_CMD_WRITE_GRAPHICS = ['E']
	COMM_CMD_WRITE_SPECIAL = ['W']
	COMM_CMD_READ_SPECIAL = ['R']

	# use these to control how a body of text is shown
	COMM_DISPLAY_MODE_AUTO = ['A']
	COMM_DISPLAY_MODE_FLASH = ['B'] 
	COMM_DISPLAY_MODE_HOLD = ['C'] 
	COMM_DISPLAY_MODE_INTERLOCK = ['D'] 
	COMM_DISPLAY_MODE_ROLLDOWN = ['E'] 
	COMM_DISPLAY_MODE_ROLLUP = ['F'] 
	COMM_DISPLAY_MODE_ROLLIN = ['G']
	COMM_DISPLAY_MODE_ROLLOUT = ['H']
	COMM_DISPLAY_MODE_ROLLLEFT = ['I']
	COMM_DISPLAY_MODE_ROLLRIGHT = ['J']
	COMM_DISPLAY_MODE_ROTATE = ['K']
	COMM_DISPLAY_MODE_SLIDE = ['L']
	COMM_DISPLAY_MODE_SNOW = ['M']
	COMM_DISPLAY_MODE_SPARKLE = ['N']
	COMM_DISPLAY_MODE_SPRAY = ['O']
	COMM_DISPLAY_MODE_STARBURST = ['P']
	COMM_DISPLAY_MODE_SWITCH = ['Q']
	COMM_DISPLAY_MODE_TWINKLE = ['R']
	COMM_DISPLAY_MODE_WIPEDOWN = ['S']
	COMM_DISPLAY_MODE_WIPEUP = ['T']
	COMM_DISPLAY_MODE_WIPEIN = ['U']
	COMM_DISPLAY_MODE_WIPEOUT = ['V']
	COMM_DISPLAY_MODE_WIPELEFT = ['W']
	COMM_DISPLAY_MODE_WIPERIGHT = ['X']
	COMM_DISPLAY_MODE_CYCLECOLOR = ['Y']
	COMM_DISPLAY_MODE_CLOCK = ['Z']

	# metadata about how to show the text
	COMM_DISPLAY_SPEED_1 = ['1']	# fastest
	COMM_DISPLAY_SPEED_2 = ['2']
 	COMM_DISPLAY_SPEED_3 = ['3']
	COMM_DISPLAY_SPEED_4 = ['4']
	COMM_DISPLAY_SPEED_5 = ['5']	#slowest
	COMM_PAUSE_TIME_2 = ['2']
	COMM_PAUSE_TIME_5 = ['5']
	COMM_PAUSE_TIME_9 = ['9']
	COMM_SHOW_DATE_NONE = ['0','0']
	COMM_START_SHOW_TIME_NONE = ['0','0','0','0']
	COMM_END_SHOW_TIME_NONE = ['2','3','5','9']
	COMM_PREPARATIVE_PLACEHOLDER = ['\xff','\xff','\xff'] 

	# how text is aligned on the sign
	COMM_ALIGN_MODE_LEFT = ['1']
	COMM_ALIGN_MODE_ALIGN = ['2']
	COMM_ALIGN_MODE_RIGHT = ['3']
	
	COMM_TEXT_DATA_FONT_SS5 = ['\xfe','A']
	COMM_TEXT_DATA_FONT_SS7 = ['\xfe','E']
	COMM_TEXT_DATA_COLOR_AUTO = ['\xfd','A']
	
	# footer constants for comms to the sign
	COMM_EFFICACY_START = ['\x00']			# checksum
	COMM_END_OF_TEXT = ['\x03']
	COMM_END_OF_TRANSMISSION = ['\x04']
		
	# special symbols supported by the sign
	COMM_SPECIAL_ASTERIX = '\xd0'
	COMM_SPECIAL_CLOCK = '\xd1'
	COMM_SPECIAL_PHONE = '\xd2'
	COMM_SPECIAL_SUNGLASSES = '\xd3'
	COMM_SPECIAL_FAUCET = '\xd4'
	COMM_SPECIAL_UKNOWN1 = '\xd5'
	COMM_SPECIAL_KEY = '\xd6'
	COMM_SPECIAL_SHIRT = '\xd7'
	COMM_SPECIAL_HELICOPTER = '\xd8'
	COMM_SPECIAL_CAR = '\xd9'
	COMM_SPECIAL_GENIE_LAMP = '\xda'
	COMM_SPECIAL_PYRAMID = '\xdb'
	COMM_SPECIAL_DUCK = '\xdc'
	COMM_SPECIAL_SCOOTER = '\xdd'
	COMM_SPECIAL_BICYCLE = '\xde'
	COMM_SPECIAL_CROWN = '\xdf'
	COMM_SPECIAL_UNKNOWN2 = '\xe0'
	COMM_SPECIAL_RIGHT_ARROW = '\xe1'
	COMM_SPECIAL_LEFT_ARROW = '\xe2'
	COMM_SPECIAL_DOWN_LEFT_ARROW = '\xe3'
	COMM_SPECIAL_UP_LEFT_ARROW = '\xe4'
	COMM_SPECIAL_TEA_CUP = '\xe5'
	COMM_SPECIAL_CHAIR = '\xe6'
	COMM_SPECIAL_HIGH_HEEL = '\xe7'
	COMM_SPECIAL_COCKTAIL = '\xe8'
	COMM_SPECIAL_ENVELOPE = '\xe9'
	COMM_SPECIAL_BELL = '\xea'
	COMM_SPECIAL_SMILEY = '\xeb'
	
	
	# Local Variables
	_serial = None     # Serial connection
	_port = None       # Name of port
	
	def __init__(self, port):
		self._port = port
		self._open()
		
	def _open(self):
		try:
			self._serial = serial.Serial(self._port, self.BAUDRATE, timeout=1)
		except portError:
			print "Error opening connection on port %s" (self._port)
	
	def _close(self):
		if self._serial:
			self._serial.close()
			
	def reset(self):
		self._close()
		self._open()
			
	# Public function outputs text to the LED Sign
	#
	# INPUTS:
	# text (string): The text you want to display
	# mode (string): The output mode, choices:
	#	"auto", "flash", "hold", "interlock", "rolldown"
	#	"rollup", "rollin", "rollout", "rollleft", "rollright",
	#	"rotate", "slide", "snow", "sparkle", "spray", "starburst"
	#	"switch", "twinkle", "wipedown", "wipreup", "wipein", 
	#	"wipeout", "wipeleft", "wiperight", "cyclecolor", "clock"
	# align (string): Text Alignment, choices:
	#	"left", "center", "right"
	# speed (string): display speed, choices:
	#	"fastest", "faster", "normal", "slow", "slower"
	# pause (integer): pause time, 2,5,9 seconds
	def writeText(self, text, mode="auto", align="center", speed="normal", pause=5):
		
		# Dictionary for inputs:
		modeLookup = {'auto': self.COMM_DISPLAY_MODE_AUTO,
					  'flash': self.COMM_DISPLAY_MODE_FLASH,
					  'hold': self.COMM_DISPLAY_MODE_HOLD,
					  'interlock': self.COMM_DISPLAY_MODE_INTERLOCK,
					  'rolldown': self.COMM_DISPLAY_MODE_ROLLDOWN,
					  'rollup': self.COMM_DISPLAY_MODE_ROLLUP,
					  'rollin': self.COMM_DISPLAY_MODE_ROLLIN,
					  'rollout': self.COMM_DISPLAY_MODE_ROLLOUT,
					  'rollleft': self.COMM_DISPLAY_MODE_ROLLLEFT,
					  'rollright': self.COMM_DISPLAY_MODE_ROLLRIGHT,
					  'rotate': self.COMM_DISPLAY_MODE_ROTATE,
					  'slide': self.COMM_DISPLAY_MODE_SLIDE,
					  'snow': self.COMM_DISPLAY_MODE_SNOW,
					  'sparkle': self.COMM_DISPLAY_MODE_SPARKLE,
					  'spray': self.COMM_DISPLAY_MODE_SPRAY,
					  'starburst': self.COMM_DISPLAY_MODE_STARBURST,
					  'switch': self.COMM_DISPLAY_MODE_SWITCH,
					  'twinkle': self.COMM_DISPLAY_MODE_TWINKLE,
					  'wipedown': self.COMM_DISPLAY_MODE_WIPEDOWN,
					  'wipeup': self.COMM_DISPLAY_MODE_WIPEUP,
					  'wipein': self.COMM_DISPLAY_MODE_WIPEIN,
					  'wipeout': self.COMM_DISPLAY_MODE_WIPEOUT,
					  'wipeleft': self.COMM_DISPLAY_MODE_WIPELEFT,
					  'wiperight': self.COMM_DISPLAY_MODE_WIPERIGHT,
					  'cyclecolor': self.COMM_DISPLAY_MODE_CYCLECOLOR,
					  'clock': self.COMM_DISPLAY_MODE_CLOCK}
		alignLookup = {'left': self.COMM_ALIGN_MODE_LEFT,
					   'center': self.COMM_ALIGN_MODE_RIGHT,
					   'right': self.COMM_ALIGN_MODE_ALIGN}
		speedLookup = {'fastest': self.COMM_DISPLAY_SPEED_1,
					   'faster': self.COMM_DISPLAY_SPEED_2,
					   'normal': self.COMM_DISPLAY_SPEED_3,
					   'slow': self.COMM_DISPLAY_SPEED_4,
					   'slower': self.COMM_DISPLAY_SPEED_5}
		pauseLookup = {2: self.COMM_PAUSE_TIME_2,
					   5: self.COMM_PAUSE_TIME_5,
					   9: self.COMM_PAUSE_TIME_9}
					   
		# Perform lookups
		if mode in modeLookup:
			mode = modeLookup[mode]
		else: 
			mode = modeLookup['auto']
		if align in alignLookup:
			align = alignLookup[align]
		else: 
			align = alignLookup['center']
		if speed in speedLookup:
			speed = speedLookup[speed]
		else:
			speed = speedLookup['normal']
		if pause in pauseLookup:
			pause = pauseLookup[pause]
		else:
			pause = pauseLookup[5]
		
		# Check to see if we have a connection
		if not self._serial:
			print "Error: No serial connection is open. Resetting Connection."
			self.reset()
			
		header = self.COMM_CMD_START + self.COMM_START_OF_HEAD + self.COMM_SEND_ADDR_PC + self.COMM_RECEIVER_ADDR_BCAST
		
		data = self.COMM_START_OF_TEXT + self.COMM_CMD_WRITE_TEXT + self.COMM_TEXT_FILE_NAME_0 + mode + speed + pause + self.COMM_SHOW_DATE_NONE + self.COMM_START_SHOW_TIME_NONE + self.COMM_END_SHOW_TIME_NONE + self.COMM_PREPARATIVE_PLACEHOLDER + align + self.COMM_TEXT_DATA_FONT_SS7 + self.COMM_TEXT_DATA_COLOR_AUTO
		for i in text.encode('ascii','ignore'):
			data.append(i)
		data += self.COMM_END_OF_TEXT
		
		footer = self.COMM_EFFICACY_START
		checksum = 0
		for i in data:
			checksum += ord(i)
		checksumStr = "%0.4X" % checksum
		for i in checksumStr:
			footer.append(i)
		footer += self.COMM_END_OF_TRANSMISSION
		
		# send the message
		try:
			self._serial.write(str(''.join(header)))
			self._serial.write(str(''.join(data)))
			self._serial.write(str(''.join(footer)))
			self._serial.flush()
			
		except transmitError:
			print "Error: Could not transmit data over serial connection."
		

	
    